# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-10-12 17:18
from __future__ import unicode_literals

import councilmatic_core.models
from django.db import migrations, models
import uuid

class Migration(migrations.Migration):

    dependencies = [
        ('councilmatic_core', '0011_auto_20161012_1204'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='bill',
            name='id',
        ),
        migrations.RemoveField(
            model_name='event',
            name='id',
        ),
        migrations.RemoveField(
            model_name='person',
            name='id',
        ),
        migrations.AlterField(
            model_name='bill',
            name='ocd_id',
            field=models.CharField(max_length=100, primary_key=True, serialize=False, unique=True),
        ),
        migrations.AlterField(
            model_name='event',
            name='ocd_id',
            field=models.CharField(max_length=100, primary_key=True, serialize=False, unique=True),
        ),
        migrations.AlterField(
            model_name='person',
            name='ocd_id',
            field=models.CharField(default=uuid.uuid4, max_length=100, primary_key=True, serialize=False, unique=True),
        ),
    ]
