{% extends "base.html" %}
{% block full_content %}
  	<div class="container-fluid">
    	<div class='col-sm-12'>
      		{% block content %}
      		{% endblock %}
    	</div>
  	</div>
  	<div class="pad"></div>

{% endblock %}