{% load static %}
<table class='table' id='council-members'>
    <thead>
        <tr>
            <th></th>
            <th>Committee Member</th>
            <th>
                {{CITY_VOCAB.MUNICIPAL_DISTRICT}}
            </th>
            <th>Role</th>
        </tr>
    </thead>
    <tbody>
        {% for membership in committee.chairs %}
            <tr>
                <td>
                    <div class="thumbnail-square">
                        <img src='{% static membership.person.headshot.url %}' alt='{{membership.person.name}}' title='{{membership.person.name}}' class='img-responsive img-thumbnail' />
                    </div>
                </td>
                <td>
                    {{membership.person.link_html|safe}}
                </td>
                <td>
                    {{membership.person.current_council_seat}}
                </td>
                <td>
                    Chairperson
                </td>
            </tr>
        {% endfor %}
        {% for membership in committee.non_chair_members %}
            <tr>
                <td>
                    <div class="thumbnail-square">
                        <img src='{% static membership.person.headshot.url %}' alt='{{membership.person.name}}' title='{{membership.person.name}}' class='img-responsive img-thumbnail' />
                    </div>
                </td>
                <td>
                    {{membership.person.link_html|safe}}
                </td>
                <td>
                    {{membership.person.current_council_seat}}
                </td>
                <td>
                    Member
                </td>
            </tr>
        {% endfor %}
    </tbody>
</table>
