# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2019-02-07 14:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('councilmatic_core', '0043_membership_extras'),
    ]

    operations = [
        migrations.AddField(
            model_name='bill',
            name='restrict_view',
            field=models.BooleanField(default=False),
        ),
    ]
